﻿using Microsoft.Data.SqlClient;
using System.Data;
using System.Data.Common;

//===========================================================
// Подробное описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//===========================================================

namespace DataGridViewColorCells.Classes
{
    internal class ConnectDatabase
    {

        private static string ConnectionString
        {
            get
            {
                var sb = new SqlConnectionStringBuilder
                {
                    DataSource = @"(LocalDB)\MSSQLLocalDB",
                    IntegratedSecurity = true,
                    InitialCatalog = "Catalog",
                    Pooling = true
                };

                return sb.ConnectionString;
            }
        }

        private static SqlDataAdapter GetAdapter(SqlConnection connection, string querystring, List<SqlParameter>? parameters = null)
        {
            SqlCommand command = new(querystring, connection);

            if (parameters != null)
            {
                foreach (SqlParameter p in parameters)
                {
                    command.Parameters.Add(p);
                }
            }

            SqlDataAdapter adapter = new()
            {
                SelectCommand = command,
                UpdateCommand = command,
                InsertCommand = command,
                DeleteCommand = command
            };
            connection.Open();

            return adapter;
        }

        public static DataSet FillDataAdapter(string querystring, List<SqlParameter>? parameters = null)
        {
            using SqlConnection connection = new(ConnectionString);
            SqlDataAdapter adapter = GetAdapter(connection, querystring, parameters);

            DataSet dataSet = new("Catalog");

            // Карта сопоставления названий таблиц набора DataSet и базы данных.
            adapter.TableMappings.Add("Table", Constants.ParentTableName);
            adapter.TableMappings.Add("Table1", Constants.ChildTableName);
            adapter.Fill(dataSet);

            return dataSet;
        }

        public static DataSet FillDataReader(string querytext, List<SqlParameter>? parameters = null)
        {
            using SqlConnection connection = new(ConnectionString); // C# 9.0
            using SqlCommand command = new(querytext, connection); // C# 9.0

            if (parameters != null)
            {
                foreach (SqlParameter p in parameters)
                {
                    command.Parameters.Add(p);
                }
            }

            connection.Open();

            List<DataTable> tables = new();

            // Получаем многие данные таблиц, в т.ч. и название таблиц автоматически.
            SqlDataReader reader = command.ExecuteReader(CommandBehavior.KeyInfo);

            while (reader.IsClosed == false)
            {
                DataTable table = new();
                table.Load(reader);

                tables.Add(table);
            }

            DataSet dataSet = new DataSet("Catalog");

            // Добавление таблиц в коллекцию набора.
            foreach (DataTable dt in tables)
            {
                dataSet.Tables.Add(dt);
            }

            return dataSet;
        }


        public static void SaveData(DataTable dt, string querystring, List<SqlParameter>? parameters = null)
        {
            using SqlConnection connection = new(ConnectionString);
            SqlDataAdapter adapter = GetAdapter(connection, querystring, parameters);

            int i = adapter.Update(dt);
            // Звук успешного обновления - для режима тестирования.
#if DEBUG
            if (i > 0) Console.Beep(3000, 100);
#endif
        }
    }
}
